"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.BullMonitor = void 0;
const data_sources_1 = require("./gql/data-sources");
const type_defs_1 = require("./gql/type-defs");
const resolvers_1 = require("./gql/resolvers");
const ui_1 = require("./ui");
const config_1 = require("./gql/data-sources/bull/config");
const metrics_collector_1 = require("./metrics-collector");
const queue_1 = require("./queue");
class BullMonitor {
    constructor(config) {
        this.gqlBasePath = '/graphql';
        this.defaultMetricsConfig = {
            redisPrefix: 'bull_monitor::metrics::',
            collectInterval: { hours: 1 },
            maxMetrics: 100,
            blacklist: [],
        };
        this.defaultConfig = {
            queues: [],
            baseUrl: '',
            gqlIntrospection: true,
            gqlPlayground: true,
            textSearchScanCount: config_1.DEFAULT_TEXT_SEARCH_SCAN_COUNT,
            metrics: false,
        };
        this.config = Object.assign(Object.assign(Object.assign({}, this.defaultConfig), config), { metrics: config.metrics
                ? Object.assign(Object.assign({}, this.defaultMetricsConfig), config.metrics) : false });
        this.ui = new ui_1.UI();
        this._queues = this.config.queues.map(queue_1.patchBullQueue);
        if (this.config.metrics) {
            this.initMetricsCollector();
        }
    }
    get queues() {
        return this._queues;
    }
    set queues(queues) {
        this._queues = queues.map(queue_1.patchBullQueue);
        if (this.metricsCollector && this.config.metrics) {
            this.metricsCollector.stopCollecting();
            this.initMetricsCollector();
        }
    }
    createServer(Server) {
        this.server = new Server({
            typeDefs: type_defs_1.typeDefs,
            // @ts-ignore
            resolvers: resolvers_1.resolvers,
            playground: this.config.gqlPlayground,
            introspection: this.config.gqlIntrospection,
            dataSources: () => ({
                bull: new data_sources_1.BullDataSource(this._queues, {
                    textSearchScanCount: this.config.textSearchScanCount,
                }),
                metrics: new data_sources_1.MetricsDataSource(this.metricsCollector),
            }),
        });
    }
    startServer() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.server.start();
        });
    }
    renderUi() {
        return this.ui.render();
    }
    get baseUrl() {
        return this.config.baseUrl;
    }
    get uiEndpoint() {
        return this.baseUrl || '/';
    }
    get gqlEndpoint() {
        const base = this.baseUrl;
        if (!base) {
            return this.gqlBasePath;
        }
        else if (base.endsWith('/')) {
            return base.slice(0, -1) + this.gqlBasePath;
        }
        return base + this.gqlBasePath;
    }
    initMetricsCollector() {
        this.metricsCollector = new metrics_collector_1.MetricsCollector(this._queues, this.config.metrics);
        this.metricsCollector.startCollecting();
    }
}
exports.BullMonitor = BullMonitor;
//# sourceMappingURL=index.js.map