"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MetricsCollector = void 0;
const json_1 = require("./services/json");
const toad_scheduler_1 = require("toad-scheduler");
class MetricsCollector {
    constructor(queues, _config) {
        this._config = _config;
        this._taskFn = () => __awaiter(this, void 0, void 0, function* () {
            const metrics = yield this._collect();
            yield this._persist(metrics);
        });
        this._scheduler = new toad_scheduler_1.ToadScheduler();
        this._queues = queues.filter((q) => !_config.blacklist.includes(q.name));
    }
    startCollecting() {
        this._maybeCreateJob();
        this._scheduler.addSimpleIntervalJob(this._job);
    }
    stopCollecting() {
        this._scheduler.stop();
    }
    extract(queue, start = 0, end = -1) {
        return __awaiter(this, void 0, void 0, function* () {
            const key = this._buildPersistKey(queue);
            const metrics = yield this._redisClient.lrange(key, start, end);
            return metrics.map(json_1.JsonService.maybeParse).filter(Boolean);
        });
    }
    clear(queue) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this._redisClient.del(this._buildPersistKey(queue));
        });
    }
    clearAll() {
        return __awaiter(this, void 0, void 0, function* () {
            const pipeline = this._redisClient.pipeline();
            this._queues.forEach((queue) => {
                pipeline.del(this._buildPersistKey(queue.id));
            });
            yield pipeline.exec();
        });
    }
    _maybeCreateJob() {
        if (!this._job) {
            const task = new toad_scheduler_1.AsyncTask('collect-metrics', this._taskFn);
            this._job = new toad_scheduler_1.SimpleIntervalJob(this._config.collectInterval, task);
        }
    }
    _collect() {
        return __awaiter(this, void 0, void 0, function* () {
            const timestamp = Date.now();
            return yield Promise.all(this._queues.map((queue) => __awaiter(this, void 0, void 0, function* () {
                return ({
                    timestamp,
                    queue: queue.id,
                    counts: yield queue.getJobCounts(),
                });
            })));
        });
    }
    _persist(metrics) {
        return __awaiter(this, void 0, void 0, function* () {
            const client = this._redisClient;
            const lpopPipeline = client.pipeline();
            yield Promise.all(metrics.map((metric) => __awaiter(this, void 0, void 0, function* () {
                const key = this._buildPersistKey(metric.queue);
                const listLen = yield client.rpush(key, JSON.stringify(metric));
                if (listLen > this._config.maxMetrics) {
                    lpopPipeline.lpop(key);
                }
            })));
            yield lpopPipeline.exec();
        });
    }
    _buildPersistKey(queue) {
        return this._config.redisPrefix + queue;
    }
    get _redisClient() {
        return this._queues[0].client;
    }
}
exports.MetricsCollector = MetricsCollector;
//# sourceMappingURL=metrics-collector.js.map